%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Checkers Detection
% Introduction to Computational and Biological Vision - Fall 2006
% Yuval Fledel & Khait Vitaly
% Ben-Gurion University
% http://www.cs.bgu.ac.il/~ben-shahar/Teaching/Computational-Vision/
%
% imaq() - No arguments, no return value
% loads the board and plot the result on screen.
%
% see the loading in get_board(). it supports capturing from a webcam and
% loading from a file.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function checkers()

%%%% Step 0: Load the image %%%%

global DEBUG;
% DEBUG=1; % load from file.
% DEBUG=0; % capture from webcam.
DEBUG= 1;

% convert to grayscale and detect edges
rgb_frame = get_board();
frame  = rgb2gray(rgb_frame);

%%%% Step 1: Detect the board orientation %%%%
BW = edge(frame,'canny');
% figure,imshow(BW);

%find lines on the board
[hough_accumulator,theta,rho] = hough(BW,'ThetaResolution',2,'RhoResolution',4);
threshold = ceil(0.3*max(hough_accumulator(:)));
peaks  = houghpeaks(hough_accumulator,50,'threshold',threshold);
lines = houghlines(BW,theta,rho,peaks,'FillGap',10,'MinLength',15);

% % Debug - Plot the detected lines on screen.
% hold on;
% for k = 1:length(lines)
%     l = lines(k); 
%    xy = [l.point1; l.point2];
%    plot(xy(:,1),xy(:,2),'LineWidth',2,'Color','yellow');
% end

% vote for the most faviourite orientation using the original
% hough transform for degrees :-)
max_deg = hough_for_degrees(lines);

% rotation angle is opposite to the board oriantation.
rotation_angle =deg2right_quadrants(-max_deg);

% show image on screen.
% show_lines(BW, lines, max_deg);

% Rotate the image back to align with the axis.
rotated_frame = im2single(imrotate(frame, rotation_angle,'bicubic','crop'));
rotated_frame_color = im2single(imrotate(rgb_frame, rotation_angle,'bicubic','crop'));

%%%% Step 2: Detect the board location in the image %%%%
[best_y, best_x1, best_dx] = find_board_width(rotated_frame);
[best_x, best_y1, best_dy] = find_board_width(rotated_frame');

% add a small border to the detected location. to compansate with noise.
border = 4; % The border width in pixels.
best_x1 = max(best_x1 - border , 1);
best_dx = min(best_dx + 2 * border,size(rotated_frame,2) - best_x1 + 1);
best_y1 = max(best_y1 - border , 1);
best_dy = min(best_dy + 2 * border,size(rotated_frame,1) - best_y1 + 1);

% Crop the image with the border.
cropped_image = rotated_frame(best_y1:best_y1+best_dy,best_x1:best_x1+best_dx);

%%%% Steps 3+4: Detect the board parity (is_upleft_white) and detect white checkers %%%%
[whites_ys, whites_xs, is_upleft_white] = identify_white_discs(cropped_image);

%%%% Step 5: Detect black checkers %%%%
[blacks_ys, blacks_xs] = findCircles(cropped_image,border);

% Remove black checkers from non-playable cells
 % find non-playable cells.
[clear_ys,clear_xs] = find((checkerboard(1)>0)~=is_upleft_white);
 % convert both subscripts to indices.
blacks = sub2ind([8 8],blacks_ys, blacks_xs);
clears = sub2ind([8 8],clear_ys, clear_xs);
 % mark as not valid.
blacks(ismember(blacks,clears))=0;
 % create a new list without the invalids.
blacks = blacks(find(blacks));
 % convert back to subscripts.
[blacks_ys, blacks_xs] = ind2sub([8 8],blacks);

%%%% Step 6: Plot the results %%%%
figure, imshow(rotated_frame_color); hold on;
plot([best_x1 best_x1+best_dx],[best_y best_y],'LineWidth',2,'Color','red');
plot([best_x best_x],[best_y1 best_y1+best_dy],'LineWidth',2,'Color','red');
plot_discs(whites_ys, whites_xs, best_y1, best_x1, size(cropped_image),border,'green');
plot_discs(blacks_ys, blacks_xs, best_y1, best_x1, size(cropped_image),border,'cyan');
% hgsave('processed_board');
% saveas(h,'processed_board.png');

%%%% Done %%%%

%%%% Subfunctions: %%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Load the board
% (Step 0)
% DEBUG: 0=Capture, 1=file.
% RGB: The 3 colored image matrix.
function [RGB]=get_board()
global DEBUG;
board_file_name= 'board5.png';

if (DEBUG)
    RGB = imread(board_file_name);
else
    [RGB]=capture_from_webcam();
    imwrite(RGB, 'board.png','png');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Capture from a webcam.
% (Part of Step 0)
% RGB: The 3 colored image matrix.
function [RGB]=capture_from_webcam()

% Create video input object. 
vid = videoinput('winvideo',1,'RGB24_320x240');
RGB = getsnapshot(vid);
% The user will want to see the image right away.
imshow(RGB);
% Clean up.
delete(vid);
clear vid

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Find the smallest possible rotation angle.
% (Part of Step 1)
% Add/Substract 90*n from angle until the result is in the [-45:45] range.
% angle: The input angle.
% rot_angle: The absolute smallest possible rotation angle.
function rot_angle=deg2right_quadrants(angle)
rot_angle = angle;
while (rot_angle>45)
    rot_angle = rot_angle - 90;
end;
while (rot_angle<-45)
    rot_angle= rot_angle + 90;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% hough transform for degrees :-)
% (Part of Step 1)
% lines (in): A vector of points from the hough for lines.
% max_deg: The most occurant deg.
function max_deg = hough_for_degrees(lines)

degs = zeros(1,380); % index is angle+190
for x = 1:length(lines)
   l=lines(x);
   p = l.point2 - l.point1;
   deg = int32(rad2deg(atan2(p(1), p(2))));
   l.theta = deg;  % update theta.
   len = norm(l.point2-l.point1);
   degs(deg+189)=degs(deg+189)+len/2;
   degs(deg+190)=degs(deg+190)+len;
   degs(deg+191)=degs(deg+191)+len/2;
end
max_deg = find(degs>=max(degs))-190;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot the lines on screen
% (A debug part of Step 1)
function show_lines(BW, lines, max_deg)

%figure, 
imshow(BW), hold on
for k = 1:length(lines)
    l = lines(k); 
   xy = [l.point1; l.point2];
   p = l.point2 - l.point1;
   deg = rad2deg(atan2(p(1), p(2)));
   if (abs(deg-max_deg)<3)
       color = 'cyan'; % Selected lines in cyan
   else
       color = 'green';% non selected in green.
   end;
   plot(xy(:,1),xy(:,2),'LineWidth',2,'Color',color);

   % Plot beginnings and ends of lines
   plot(xy(1,1),xy(1,2),'x','LineWidth',2,'Color','yellow');
   plot(xy(2,1),xy(2,2),'x','LineWidth',2,'Color','red');   
end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Find the horizontal board location.
% If you want the vertical, use the transposed image.
% (Step 2)
% rotated_frame: the aligned image matrix.
% best_y: The row of the best matched line.
% best_x1: The left column of the best matched line.
% best_x2: The right column of the best matched line.
function [best_y, best_x1, best_dx] = find_board_width(rotated_frame)

% Precomputation: Store alternating lines in variable lengths.
scan_line_length = size(rotated_frame,2);
storage = cell(1, floor(scan_line_length/8));
for i = 1:1:scan_line_length/8
  checkers_line = (mod(0:i*8-1 , i*2)>(i-1)); %generate board line of length i*8
  checkers_line =(checkers_line)*2 -1; %black=-0.5, white=0.5
  storage(i) = {checkers_line};
end;

% Find the best match.
best_val = 0;
best_y = 1;
best_x1 = 1;
best_dx = 1;
for i =1:size(rotated_frame,1)
    % Take the current line from the image.
    % Convert it to the [-1,1] scale.
   scan_line = (rotated_frame(i,:)>0.5)*2-1;
   for dx = 1:scan_line_length/8
       % Take the -1,1,-1,1,-1,1,-1 line from the precomputation
       checkers_line = storage{dx};
       for x1 = 1:4:scan_line_length-dx*8
           % Compute the matching value.
           val = sum(scan_line(x1:x1 + dx*8-1) .*checkers_line);
           % Only store the best value.
           if (best_val<abs(val))
               best_val = abs(val);
               best_y = i;
               best_x1 = x1;
               best_dx = dx*8;
               % debug - Plot the steps in yellow.
               % plot([best_x1 best_x1+best_dx],[best_y best_y],'LineWidth',2,'Color','yellow');
           end;
       end;
   end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Find the white discs in the image.
% (Steps 3+4)
% cropped_image: the aligned cropped image matrix.
% is_upleft_white: true if the topleft cell is non-playable.
%                         false if playable. (output of step 3)
% whites_y: a vector with the row subscripts of the discs. (step 4)
% whites_x: a vector with the column subscripts of the discs. (step 4)
function [whites_y, whites_x, is_upleft_white] = identify_white_discs(cropped_image)

cropped_size = size(cropped_image);
% Initialize an accumulator.
val = zeros(8,8);
for cx=0:7
    % Get the 'x' coordinates of the inner third of each cell.
    x1=round(cx*cropped_size(2)/8+cropped_size(2)/24);
    x2=round(x1+cropped_size(2)/24);
    for cy=0:7
       % Get the 'y' coordinates of the inner third of each cell.
       y1=round(cy*cropped_size(1)/8+cropped_size(1)/24);
       y2=round(y1+cropped_size(1)/24);
       % Crop only the inner ninth of each cell.
       cropped_square = cropped_image(y1:y2, x1:x2);
       % Insert the light intensity in this square into the accumulator.
       val(cy+1,cx+1) = sum(cropped_square(:));
    end;
end;

% We have 2 sets of cells. Partition the light intensity into 2 matrices.
upleft_corner_squares = [val(1,[1 3 5 7]); val(2,[2 4 6 8]); val(3,[1 3 5 7]); val(4,[2 4 6 8]); val(5,[1 3 5 7]); val(6,[2 4 6 8]); val(7,[1 3 5 7]); val(8,[2 4 6 8])];
non_upleft_corner_squares = [val(1,[2 4 6 8]); val(2,[1 3 5 7]); val(3,[2 4 6 8]); val(4,[1 3 5 7]); val(5,[2 4 6 8]); val(6,[1 3 5 7]); val(7,[2 4 6 8]); val(8,[1 3 5 7])];

% Sum up, and the lighter wins.
is_upleft_white = sum(sum(upleft_corner_squares))>sum(sum(non_upleft_corner_squares));
% A conviniance if.
if (is_upleft_white)
    filled_squares = non_upleft_corner_squares;
    non_filled_squares = upleft_corner_squares;
else
    filled_squares = upleft_corner_squares;
    non_filled_squares = non_upleft_corner_squares;
end;

% Substract one from the other.
% This has the effect of compansating local light noise.
a = non_filled_squares - filled_squares;
% figure, imagesc(a);

% Use a threshold of max/4.
a = a-min(min(a));
[whites_y, x] = find(a<max(max(a))/4);
% Compute the 'x' coordinates, remember that we have a parity to consider.
whites_x = x*2 - is_upleft_white*(1-mod(whites_y,2)) - (1-is_upleft_white)*mod(whites_y,2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot circles on the image.
% (Part of step 6)
% y_vec: a vector with the row subscripts of the discs.
% x_vec: a vector with the column subscripts of the discs.
% y1: The top row of the board (including border)
% x1: The left column of the board (including border)
% cropped_size: The size of the displayed image (including border)
% border: The border width in pixels
% color: The discs color.
function plot_discs(y_vec, x_vec, y1, x1, cropped_size,border,color);

cropped_size = cropped_size - [2*border 2*border];
x1 = x1 + border;
y1 = y1 + border;

% for each disc:
for i=1:size(x_vec,1)
    % the radius.
    r = cropped_size(2)/16;
    % the center of the disc.
    xc=round(x_vec(i)*cropped_size(2)/8-cropped_size(2)/16)+x1;
    yc=round(y_vec(i)*cropped_size(1)/8-cropped_size(1)/16)+y1;

    % plot 360 pixels. This should be more than enough.
    xs = xc+r*cos([1:360]);
    ys = yc+r*sin([1:360]);
    hold on; % plot on the existing figure.
    plot( xs , ys,'Color',color );
end;

