function [blacks_ys, blacks_xs]=findCircles(inputImage,border)

% basic thresholding for prewitt edge detector
edgeThreshold = 0.09;
circleThreshold = 15;
% circleThreshold = 0;

% limit the size of the circles to improve the efficiency
[sizeY,sizeX]=size(inputImage);
minRadius=floor(min(sizeY,sizeX)/20);
maxRadius=floor(min(sizeY,sizeX)/14);

% initialize accumulator
accumulator=zeros(sizeY,sizeX,maxRadius);

% find gradients magnitudes and directions
[magnitudes,orientations]=analyzePrewit(inputImage);

% decrease the number of detected edge points
% by thresholding and removing the points
% belonging to vertical and horizontal lines
cleanedPoints = (magnitudes>edgeThreshold);
cleanedPoints = cleanedPoints & (abs(orientations)>pi/36) ;
cleanedPoints = cleanedPoints & (abs(abs(orientations)-pi/2)>pi/36);

% vote for all points
[pR,pC]=ind2sub(size(magnitudes),find(cleanedPoints>0));
for i=1:2:size(pR,1)        
    accumulator=updateAccumulatorWithPoint(accumulator,pR(i),pC(i),orientations(pR(i),pC(i)),minRadius:maxRadius);
end

% figure, imshow(inputImage), hold on;
[cy,cx,r]=ind2sub(size(accumulator),find(accumulator>circleThreshold));

% initialize cells accumulator
cell_votes = zeros(8,8);
cell_size = size(inputImage)/8;
% vote for cells
for i=1:size(cy)
    cur_cell_y = min(max(floor((cy(i)-border)/cell_size(1))+1,1),8); 
    cur_cell_x = min(max(floor((cx(i)-border)/cell_size(2))+1,1),8);
    weight = accumulator(cy(i),cx(i),r(i));
    cell_votes(cur_cell_y ,cur_cell_x) = cell_votes(cur_cell_y ,cur_cell_x) + weight;
end;

% create histogram bins
max_vote = max(cell_votes(:));
min_vote = min(cell_votes(:));
hist_bins = min_vote:(max_vote-min_vote)/100:max_vote;
[n,x] = hist(cell_votes(:), hist_bins );
figure, hist(cell_votes(:), hist_bins );

% clusterize histogram into 2 clusters by weighted rule
cluster_map = cluster( linkage(pdist([1:size(n,2); n]'),'weighted'),2);
first_cluster_size = nnz(cluster_map==cluster_map(1));

% threshold equals to the value of the smallest element
% of the second cluster
threshold = x(first_cluster_size+1);

[blacks_ys, blacks_xs] = find(cell_votes>=threshold);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% perform voting for a single point
function accumulator=updateAccumulatorWithPoint(accumulator,y,x,theta,radiuses)

%calculate the direction of the edge 
if (theta>0)
    result_theta = pi/2 - theta;
else
    result_theta = theta + pi/2;
end

% compute coordinates of center for each radius
cy=y-radiuses*cos(result_theta);
cx=x + radiuses*sin(result_theta);
vectorR=floor(radiuses);
cx = floor(cx);
cy = floor(cy);

[sY,sX,R]=size(accumulator);

% vote
accumulator = voterForCell(accumulator, cx, cy, vectorR, sX, sY);
accumulator = voterForCell(accumulator, cx, cy+1, vectorR, sX, sY);
accumulator = voterForCell(accumulator, cx+1, cy, vectorR, sX, sY);
accumulator = voterForCell(accumulator, cx+1, cy+1, vectorR, sX, sY);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% vote for cell in specified coordinates
function accumulator = voterForCell(accumulator, x, y, r, sizeX, sizeY)
fittingInd=((y<=sizeY) & (x<=sizeX) & (y>=1) & (x>=1));
i=find(fittingInd>0);
accumulator(y(i),x(i),r(i))=accumulator(y(i),x(i),r(i))+1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the directions and magnituded of gradients for each pixel in the
% image
function [magnitudes,orientations]=analyzePrewit(inputImage)
nImage=im2single(inputImage);
delta1=[-1,0,1;-1,0,1;-1,0,1]/6;
delta2=delta1';                                            
p1=conv2(nImage,delta1,'same');
p2=conv2(nImage,delta2,'same');
magnitudes=sqrt(p1.^2+p2.^2);        
orientations=(atan2(p2,p1));